@@ -1,9 +1,6 @@
-import numpy as np
-from scipy.fft import fft, ifft
-from . import helperfunctions as hf
+from .libraries import *
 from . import config
-from twiggy import quick_setup, log
-import time  # used for setting random number generator seed if None
+from . import helperfunctions as hf
 
 quick_setup()
 logger = log.name('lossmodel')
@@ -113,6 +110,7 @@ class _Severity:
                   deductible plus cover is the contract upper priority or severity 'exit point'
                   (default value is infinity).
     :type cover: ``int`` or ``float``
+
     :param \\**kwargs:
         See below
 
@@ -121,7 +119,7 @@ class _Severity:
           Name of the severity distribution.
           See the distribution module for the continuous distributions supported in GEMAct.
         * *sev_par* (``dict``) --
-          parameters of the severity distribution.
+          Parameters of the severity distribution.
     """
 
     def __init__(
@@ -344,7 +342,6 @@ class _Severity:
 
         :return: discrete severity, nodes sequence and discrete probabilities.
         :rtype: ``dict``
-
         """
 
         last_node_prob = self._upper_discr_point_prob_adjuster()
@@ -379,8 +376,11 @@ class LossModel(_Severity, _Frequency):
                               Namely, ``aggr_n_deductible * deductible`` =  aggregate priority.
     :type aggr_n_deductible: ``int``
     :param n_reinst: number of reinstatements layers.
-                     Alternative parametrization to aggregate limit cover,
-                     i.e. aggregate cover = number of reinstatement * cover
+                     Alternative parametrization to aggregate cover (a.k.a. aggregate limit),
+                     namely aggregate cover = (number of reinstatement +1) * cover.
+                     E.g. When the number of reinstatements = 0, the aggregate cover is equal to the cover,
+                     when number of reinstatements is infinity there is no aggregate cover
+                     (the aggregate cover is infinity).
     :type n_reinst: ``int``
     :param reinst_loading: reinstatements layers loading.
     :type reinst_loading: ``float``
@@ -396,6 +396,7 @@ class LossModel(_Severity, _Frequency):
     :type random_state: ``int``, optional
     :param n_aggr_dist_nodes: number of nodes in the approximated aggregate loss distribution.
     :type n_aggr_dist_nodes: ``int``
+
     :param \\**kwargs:
         See below
 
@@ -417,7 +418,7 @@ class LossModel(_Severity, _Frequency):
         * *deductible* (``int`` or ``float``) --
           Contract deductible, also referred to as retention or priority (default value is 0).
         * *cover* (``int`` or ``float``) --
-          Contract cover (deductible plus cover is the contract upper priority or severity 'exit point').
+          Contract cover, also referred to as contract limit. Namely cover plus deductible is the contract upper priority or severity 'exit point'.
     """
 
     def __init__(
@@ -428,7 +429,7 @@ class LossModel(_Severity, _Frequency):
             tilt_value=0,
             random_state=None,
             n_aggr_dist_nodes=20000,
-            aggr_n_deductible=1,
+            aggr_deductible=0,
             n_reinst=float('inf'),
             reinst_loading=0,
             alpha_qs=1,
@@ -442,7 +443,7 @@ class LossModel(_Severity, _Frequency):
         # Loss Model contract features
         self.__aggr_loss_dist = None
         self.fq_model.par_franchise_adjuster(self.nu)
-        self.aggr_n_deductible = aggr_n_deductible
+        self.aggr_deductible = aggr_deductible
         self.n_reinst = n_reinst
         self.reinst_loading = reinst_loading
         self.alpha_qs = alpha_qs
@@ -531,14 +532,15 @@ class LossModel(_Severity, _Frequency):
         self.__tilt_value = value
 
     @property
-    def aggr_n_deductible(self):
-        return self.__aggr_n_deductible
+    def aggr_deductible(self):
+        # aggregate deductible or priority
+        return self.__aggr_deductible
 
-    @aggr_n_deductible.setter
-    def aggr_n_deductible(self, value):
-        assert isinstance(value, (int, float)), 'Aggregate number of deductible value should be an integer or a float'
-        assert value >= 0, 'Aggregate number of deductible must be positive'
-        self.__aggr_n_deductible = value
+    @aggr_deductible.setter
+    def aggr_deductible(self, value):
+        assert isinstance(value, (int, float)), 'Aggregate deductible value should be an integer or a float'
+        assert value >= 0, 'Aggregate deductible must be positive'
+        self.__aggr_deductible = value
 
     @property
     def n_reinst(self):
@@ -594,11 +596,6 @@ class LossModel(_Severity, _Frequency):
         # aggregate cover or limit
         return (self.n_reinst + 1) * self.cover
 
-    @property
-    def aggr_deductible(self):
-        # aggregate deductible or priority
-        return self.aggr_n_deductible * self.deductible
-
     @property
     def nu(self):
         return 1 - self.sev_model.cdf(self.deductible)
@@ -641,6 +638,8 @@ class LossModel(_Severity, _Frequency):
 
         """
 
+        assert self.aggr_deductible >= self.deductible, logger.error('Deductible cannot exceed aggregate deductible')
+
         if (aggr_loss_dist_method is None) and (self.aggr_loss_dist_method is None):
             output = {'nodes': None, 'epdf': None, 'ecdf': None}
         else:
@@ -813,27 +812,15 @@ class LossModel(_Severity, _Frequency):
         Aggregate loss distribution percent point function, a.k.a. the quantile function,
         inverse of the cumulative distribution function.
 
-        :param q: vector of probabilities.
+        :param q: probability.
         :type q: ``float`` or ``numpy.ndarray``
-        :return: vector of quantiles.
-        :rtype: ``numpy.ndarray``
-
+        :return: quantile.
+        :rtype: ``numpy.float64`` or ``numpy.ndarray``
         """
 
         self._aggr_loss_dist_check()
-
-        try:
-            if self.aggr_loss_dist_method != 'mc':
-                q_ = np.array([])
-                q = list(q)
-                for i in q:
-                    q_ = np.append(q_, [self.aggr_loss_dist['nodes'][self.aggr_loss_dist['ecdf'] >= i][0]])
-                return q_
-            else:
-                return np.quantile(self.aggr_loss_dist['nodes'], q=q)
-        except Exception:
-            logger.error('Please make sure q is a list')
-            raise
+        q = np.ravel(q)
+        return np.quantile(self.aggr_loss_dist['nodes'], q=q)
 
     def aggr_loss_cdf(self, x):
         """
@@ -847,9 +834,10 @@ class LossModel(_Severity, _Frequency):
 
         self._aggr_loss_dist_check()
 
+        x = np.ravel(x)
         x = np.maximum(x, 0)
-        y_ = np.concatenate((0, self.aggr_loss_dist['ecdf']))
-        x_ = np.concatenate((0, self.aggr_loss_dist['nodes']))
+        y_ = np.append(0, self.aggr_loss_dist['ecdf'])
+        x_ = np.append(0, self.aggr_loss_dist['nodes'])
         output = np.empty(len(x))
 
         for k in np.arange(len(x)):
@@ -872,21 +860,21 @@ class LossModel(_Severity, _Frequency):
         :return: random variates.
         :rtype: ``numpy.int`` or ``numpy.ndarray``
         """
+
         self._aggr_loss_dist_check()
 
         random_state = int(time.time()) if random_state is None else random_state
         assert isinstance(random_state, int), logger.error("Please make sure random_state is as an integer")
+        np.random.seed(random_state)
 
+        assert (size > 0), logger.error("Size must be > 0")
         try:
             size = int(size)
         except Exception:
             logger.error('Please make sure size is an integer')
             raise
 
-        np.random.seed(random_state)
-
         output = np.random.choice(self.aggr_loss_dist['nodes'], size=size, p=self.aggr_loss_dist['epdf'])
-
         return output
 
     def aggr_loss_mean(self):
@@ -923,7 +911,6 @@ class LossModel(_Severity, _Frequency):
         :param t: deductible.
         :type t: ``float``
         :return: stop loss contract expected value.
-
         """
         t = np.repeat([t], 1)
         x = self.aggr_loss_dist['nodes']
@@ -956,13 +943,13 @@ class LossModel(_Severity, _Frequency):
         :rtupe: ``numpy.ndarray``
         """
         output = self._stop_loss_pricing(self.aggr_deductible) - self._stop_loss_pricing(
-            self.aggr_deductible + (self.n_reinst + 1) * self.exit_point)
+            self.aggr_deductible + (self.n_reinst + 1) * self.cover)
         if self.n_reinst > 0:
             lower_k = np.linspace(start=0, stop=self.n_reinst, num=self.n_reinst + 1)
             dlk = (self._stop_loss_pricing(
-                self.aggr_deductible + lower_k[:-1] * self.exit_point) - self._stop_loss_pricing(
-                self.aggr_deductible + lower_k[1:] * self.exit_point))
-            den = 1 + np.sum(dlk * self.reinst_loading) / self.exit_point
+                self.aggr_deductible + lower_k[:-1] * self.cover) - self._stop_loss_pricing(
+                self.aggr_deductible + lower_k[1:] * self.cover))
+            den = 1 + np.sum(dlk * self.reinst_loading) / self.cover
             output = output / den
         return output
 
@@ -984,26 +971,29 @@ class LossModel(_Severity, _Frequency):
 
         data = [
             ['Deductible', 'd', self.deductible],
-            ['Cover', 'u - d', self.exit_point - self.deductible],
-            ['Upper priority', 'u', self.exit_point],
-            ['Aggregate deductible', 'L', self.aggr_deductible],
-            ['Quota share ceded portion', 'alpha', self.alpha_qs]
+            ['Cover', 'c', self.exit_point - self.deductible],
+            ['Upper priority', 'c+d', self.exit_point],
+            ['Aggregate deductible', 'm', self.aggr_deductible],
+            ['Quota share portion', 'a', self.alpha_qs]
         ]
         if self.n_reinst != float('inf'):
-            data.extend([['Number of reinstatements', 'K', self.n_reinst]])
+            data.extend([['Reinstatements (no.)', 'k', self.n_reinst]])
 
         data.append(['Pure premium', 'P', self.alpha_qs * p_])
         print('{: >20} {: >20} {: >20} {: >20}'.format(' ', *['Contract specification', 'parameter', 'value']))
-        print('{: >20} {: >20}'.format(' ', *[' ==================================================================']))
+        print('{: >20} {: >20}'.format(' ', *['====================================================================']))
         for row in data:
             print('{: >20} {: >20} {: >20} {: >20}'.format('', *row))
-        print('\n Reinstatement layer loading c: ', self.reinst_loading)
+        print('\n Reinstatement layer loading l: ', self.reinst_loading)
         if self.aggr_loss_dist_method == 'mc':
             print(self.aggr_loss_dist_method, '\t n_sim: ', self.n_sim, '\t random_state:', self.random_state)
         else:
+            n_sev2print = self.n_sev_discr_nodes
+            if self.cover != float('inf'):
+                n_sev2print += 1
             print(
                 self.aggr_loss_dist_method, '\t n_sev_discr_nodes m: ',
-                self.n_sev_discr_nodes, '\t n_aggr_dist_nodes n: ',
+                n_sev2print, '\t n_aggr_dist_nodes n: ',
                 self.n_aggr_dist_nodes)
 
     def print_aggr_loss_specs(self):
@@ -1030,13 +1020,18 @@ class LossModel(_Severity, _Frequency):
                 ['tilt_value', self.tilt_value],
             ])
 
+        n_sev2print = self.n_sev_discr_nodes
+
+        if self.cover != float('inf'):
+            n_sev2print += 1
+
         data.extend([
             ['sev_discr_method', self.sev_discr_method],
             ['sev_discr_step', self.sev_discr_step],
-            ['n_sev_discr_nodes', self.n_sev_discr_nodes]
+            ['n_sev_discr_nodes', n_sev2print]
         ])
 
-        print('{: >20} {: >20} {: >20}'.format(' ', *['Contract specification', 'value']))
+        print('{: >20} {: >20} {: >20}'.format(' ', *['         ', 'value']))
         print('{: >20} {: >20}'.format(' ', *[' ==================================================================']))
         for row in data:
             print('{: >20} {: >20} {: >20}'.format('', *row))
@@ -1050,20 +1045,19 @@ class LossModel(_Severity, _Frequency):
         :rtype: None
         """
         data = [
-            ['deductible', self.deductible],
-            ['cover', self.cover],
-            ['aggr_deductible', self.aggr_deductible],
-            ['alpha_qs', self.alpha_qs],
-            ['n_reinst', self.n_reinst]
+            ['Deductible', self.deductible],
+            ['Cover', self.cover],
+            ['Aggregate deductible', self.aggr_deductible],
+            ['Quota share portion', self.alpha_qs],
+            ['Reinstatements (no.)', self.n_reinst]
         ]
-        if self.reinst_loading.size > 0:
-            data.extend(['reinst_loading', self.reinst_loading])
+
 
         print('{: >20} {: >20} {: >20}'.format(' ', *['Contract specification', 'value']))
         print('{: >20} {: >20}'.format(' ', *[' ==================================================================']))
         for row in data:
-            print('{: >20} {: >20} {: >20}'.format('', *row))
-
+            print('{: >20} {: >20} {: >20}'.format(' ', *row))
+        print('\n Reinstatement layer loading c: ', self.reinst_loading)
         return
 
     def _aggr_loss_dist_check(self):
